class MERBillableCosts(PWReportPage):
	def init(self):
		super(MERBillableCosts, self).init()
		self.project				= None
		self.plannedWorkFormatter   = None
		self.actualWorkFormatter    = None		
		self.withObjective			= True
		self.contentMode			= 0
		self.withParent				= True
		self.withElement			= True
		self.withType				= True
		self.iterIndex				= 0
		self.deeperIterIndex        = 0
		self.iterItem				= None
		self.iterElement 			= None
		self.elementsSortDescriptors		= [NSSortDescriptor.alloc().initWithKey_ascending_("title", True)]
		return self
		

	def showPlannedValues(self):
		return self.contentMode == 0
		
	def showActualValues(self):
		return self.contentMode == 1

	def showObjective(self):
		return self.project.itemDescription()

	def isBillable(self):
		return self.iterItem.isBillable()

	def showActualCostsOrTakePlanned(self):
		if self.iterItem.actualAmount() != None:
			return self.iterItem.actualAmount()
		else:
			return self.iterItem.amount()

	def allElements(self):
		predicate = NSPredicate.predicateWithFormat_("isPrivate = FALSE")
		allElements = self.project.deepElements().allObjects().filteredArrayUsingPredicate_(predicate)
		return allElements.sortedArrayUsingDescriptors_(self.elementsSortDescriptors)	

	def totalBillableCosts(self, property):
		ourElements = self.allElements()
		result = 0
		if ourElements != None:
			for anElement in ourElements:
				costs = anElement.valueForKey_("sortedCosts")
				if costs != None:
					for cost in costs:
						billable = cost.valueForKey_("isBillable")
						if billable:
							amount = cost.valueForKey_(property)
							if amount != None:
								result = result + amount
							else:
								if property == "actualAmount":
									plannedAmount = cost.valueForKey_("amount")
									if plannedAmount != None:
										result = result + plannedAmount	
								
			return result	
	
	
	def plannedTotalBillableCosts(self):
		return self.totalBillableCosts("amount")
		
	def actualTotalBillableCosts(self):
		return self.totalBillableCosts("actualAmount")
		
				
	def eventCostKinds(self):
		amountPerKind = {}
		result = NSMutableArray.array()
		ourEvents = self.allElements()
		if ourEvents != None:
			for anEvent in ourEvents:
				costs = anEvent.valueForKey_("sortedCosts")
				if costs != None:
					for cost in costs:
						amount = cost.valueForKey_("amount")
						kind = cost.valueForKey_("kind")
						if kind != None:
							kindTitle = kind.valueForKey_("title")
							oldAmount = amountPerKind.get(kindTitle)
							if oldAmount == None:
								amountPerKind[kindTitle] = amount
							else:
								amountPerKind[kindTitle] = amount + oldAmount
			for key in amountPerKind.keys():
				dict = {}
				dict["name"] = key
				dict["value"] = amountPerKind[key]
				result.append(dict)
			byName = NSSortDescriptor.alloc().initWithKey_ascending_("name", True)		
			return result.sortedArrayUsingDescriptors_([byName])				

		
	


	def rowClass(self):
		if self.iterIndex%2:
			return "even"

	def totalRowClass(self):
		if len(self.allElements())%2:
			return "even"

	def dateFormat(self):
		return "medium/short"


	def cellClassForAmount(self, amount):
		if amount < 0:
			return "negativeBalance"
		else:
			return "positiveBalance"


